@@ -1,7 +1,8 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 # Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/>
+# Copyright (c) 2012-2016 The PyWavelets Developers
+#                         <https://github.com/PyWavelets/pywt>
 # See COPYING for license details.
 
 """
@@ -53,7 +54,7 @@ import pywt
 def image2array(image):
     """PIL Image to NumPy array"""
     assert image.mode in ('L', 'RGB', 'CMYK')
-    arr = numpy.fromstring(image.tostring(), numpy.uint8)
+    arr = numpy.fromstring(image.tobytes(), numpy.uint8)
     arr.shape = (image.size[1], image.size[0], len(image.getbands()))
     return arr.swapaxes(0, 2).swapaxes(1, 2).astype(numpy.float32)
 
@@ -64,7 +65,7 @@ def array2image(arr, mode):
     arr[arr < 0] = 0
     arr[arr > 255] = 255
     arr = numpy.fix(arr).astype(numpy.uint8)
-    return Image.fromstring(mode, arr.shape[1::-1], arr.tostring())
+    return Image.frombytes(mode, arr.shape[1::-1], arr.tobytes())
 
 
 def load_image(path, mode=None, size=None):
@@ -86,7 +87,7 @@ def load_image(path, mode=None, size=None):
     return im
 
 
-def blend_images(base, texture, wavelet, level, mode='sp1', base_gain=None,
+def blend_images(base, texture, wavelet, level, mode='smooth', base_gain=None,
                  texture_gain=None):
     """Blend loaded images at `level` of granularity using `wavelet`"""
 
@@ -150,7 +151,8 @@ def main():
                       type="int", default=4,
                       help="decomposition level [default: %default]")
     parser.add_option("-m", "--mode", dest="mode", metavar="MODE",
-                      default='sym', help="decomposition mode. Adjust this if"
+                      default='symmetric',
+                      help="decomposition mode. Adjust this if"
                       " getting edge artifacts [default: %default]")
     parser.add_option("-x", "--base_gain", dest="base_gain", metavar="BG",
                       type="float", default=None,
