@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/> 
+/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/>
  * Copyright (c) 2012-2016 The PyWavelets Developers
  *                         <https://github.com/PyWavelets/pywt>
  * See COPYING for license details.
@@ -42,6 +42,8 @@ int is_discrete_wavelet(WAVELET_NAME name)
             return 0;
         case CMOR:
             return 0;
+        default:
+            return -1;
     }
 
 }
@@ -350,7 +352,7 @@ DiscreteWavelet* discrete_wavelet(WAVELET_NAME name, unsigned int order)
     return w;
 }
 
-ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
+ContinuousWavelet* continuous_wavelet(WAVELET_NAME name, unsigned int order)
 {
     ContinuousWavelet *w;
     switch(name){
@@ -358,7 +360,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
         case GAUS:
             if (order > 8)
                 return NULL;
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -379,7 +381,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
             w->fbsp_order = 0;
             break;
         case MEXH:
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -397,7 +399,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
             w->fbsp_order = 0;
             break;
         case MORL:
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -417,7 +419,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
         case CGAU:
             if (order > 8)
                 return NULL;
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -439,7 +441,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
             break;
         case SHAN:
 
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -458,7 +460,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
             break;
         case FBSP:
 
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -477,7 +479,7 @@ ContinuousWavelet* continous_wavelet(WAVELET_NAME name, unsigned int order)
             break;
         case CMOR:
 
-            w = blank_continous_wavelet();
+            w = blank_continuous_wavelet();
             if(w == NULL) return NULL;
 
             w->base.support_width = -1;
@@ -559,7 +561,7 @@ DiscreteWavelet* blank_discrete_wavelet(size_t filters_length)
     return w;
 }
 
-ContinuousWavelet* blank_continous_wavelet()
+ContinuousWavelet* blank_continuous_wavelet(void)
 {
     ContinuousWavelet* w;
 
@@ -582,9 +584,6 @@ DiscreteWavelet* copy_discrete_wavelet(DiscreteWavelet* base)
 
     if(base == NULL) return NULL;
 
-    if(base->dec_len < 0 || base->rec_len < 0)
-        return NULL;
-
     w = wtmalloc(sizeof(DiscreteWavelet));
     if(w == NULL) return NULL;
 
@@ -670,7 +669,7 @@ void free_discrete_wavelet(DiscreteWavelet *w){
     wtfree(w);
 }
 
-void free_continous_wavelet(ContinuousWavelet *w){
+void free_continuous_wavelet(ContinuousWavelet *w){
 
     /* finally free struct */
     wtfree(w);
