@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/> 
+/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/>
  * Copyright (c) 2012-2016 The PyWavelets Developers
  *                         <https://github.com/PyWavelets/pywt>
  * See COPYING for license details.
@@ -10,6 +10,10 @@
 #error TYPE must be defined here.
 #else
 
+#ifndef REAL_TYPE
+#error REAL_TYPE must be defined here.
+#else
+
 #include "wt.h"
 
 #if defined _MSC_VER
@@ -36,7 +40,7 @@ int CAT(TYPE, _downcoef_axis)(const TYPE * const restrict input, const ArrayInfo
     if (input_info.ndim != output_info.ndim)
         return 1;
     if (axis >= input_info.ndim)
-        return 1;
+        return 2;
 
     for (i = 0; i < input_info.ndim; ++i){
         if (i == axis){
@@ -44,17 +48,17 @@ int CAT(TYPE, _downcoef_axis)(const TYPE * const restrict input, const ArrayInfo
             case DWT_TRANSFORM:
                 if (dwt_buffer_length(input_info.shape[i], wavelet->dec_len,
                                       dwt_mode) != output_info.shape[i])
-                    return 1;
+                    return 3;
                 break;
             case SWT_TRANSFORM:
                 if (swt_buffer_length(input_info.shape[i])
                     != output_info.shape[i])
-                    return 1;
+                    return 4;
                 break;
             }
         } else {
             if (input_info.shape[i] != output_info.shape[i])
-                return 1;
+                return 5;
         }
     }
 
@@ -160,7 +164,7 @@ int CAT(TYPE, _downcoef_axis)(const TYPE * const restrict input, const ArrayInfo
  cleanup:
     free(temp_input);
     free(temp_output);
-    return 2;
+    return 6;
 }
 
 
@@ -275,7 +279,7 @@ int CAT(TYPE, _idwt_axis)(const TYPE * const restrict coefs_a, const ArrayInfo *
                 : (const TYPE *)((const char *) coefs_a + a_offset);
             CAT(TYPE, _upsampling_convolution_valid_sf)
                 (a_row, a_info->shape[axis],
-                 wavelet->CAT(rec_lo_, TYPE), wavelet->rec_len,
+                 wavelet->CAT(rec_lo_, REAL_TYPE), wavelet->rec_len,
                  output_row, output_info.shape[axis],
                  mode);
         }
@@ -285,7 +289,7 @@ int CAT(TYPE, _idwt_axis)(const TYPE * const restrict coefs_a, const ArrayInfo *
                 : (const TYPE *)((const char *) coefs_d + d_offset);
             CAT(TYPE, _upsampling_convolution_valid_sf)
                 (d_row, d_info->shape[axis],
-                 wavelet->CAT(rec_hi_, TYPE), wavelet->rec_len,
+                 wavelet->CAT(rec_hi_, REAL_TYPE), wavelet->rec_len,
                  output_row, output_info.shape[axis],
                  mode);
         }
@@ -322,7 +326,7 @@ int CAT(TYPE, _dec_a)(const TYPE * const restrict input, const size_t input_len,
     }
 
     return CAT(TYPE, _downsampling_convolution)(input, input_len,
-                                                wavelet->CAT(dec_lo_, TYPE),
+                                                wavelet->CAT(dec_lo_, REAL_TYPE),
                                                 wavelet->dec_len, output,
                                                 2, mode);
 }
@@ -340,7 +344,7 @@ int CAT(TYPE, _dec_d)(const TYPE * const restrict input, const size_t input_len,
         return -1;
 
     return CAT(TYPE, _downsampling_convolution)(input, input_len,
-                                                wavelet->CAT(dec_hi_, TYPE),
+                                                wavelet->CAT(dec_hi_, REAL_TYPE),
                                                 wavelet->dec_len, output,
                                                 2, mode);
 }
@@ -357,7 +361,7 @@ int CAT(TYPE, _rec_a)(const TYPE * const restrict coeffs_a, const size_t coeffs_
         return -1;
 
     return CAT(TYPE, _upsampling_convolution_full)(coeffs_a, coeffs_len,
-                                                   wavelet->CAT(rec_lo_, TYPE),
+                                                   wavelet->CAT(rec_lo_, REAL_TYPE),
                                                    wavelet->rec_len, output,
                                                    output_len);
 }
@@ -374,7 +378,7 @@ int CAT(TYPE, _rec_d)(const TYPE * const restrict coeffs_d, const size_t coeffs_
         return -1;
 
     return CAT(TYPE, _upsampling_convolution_full)(coeffs_d, coeffs_len,
-                                                   wavelet->CAT(rec_hi_, TYPE),
+                                                   wavelet->CAT(rec_hi_, REAL_TYPE),
                                                    wavelet->rec_len, output,
                                                    output_len);
 }
@@ -415,7 +419,7 @@ int CAT(TYPE, _idwt)(const TYPE * const restrict coeffs_a, const size_t coeffs_a
     /* reconstruct approximation coeffs with lowpass reconstruction filter */
     if(coeffs_a){
         if(CAT(TYPE, _upsampling_convolution_valid_sf)(coeffs_a, input_len,
-                                                  wavelet->CAT(rec_lo_, TYPE),
+                                                  wavelet->CAT(rec_lo_, REAL_TYPE),
                                                   wavelet->rec_len, output,
                                                   output_len, mode) < 0){
             goto error;
@@ -427,7 +431,7 @@ int CAT(TYPE, _idwt)(const TYPE * const restrict coeffs_a, const size_t coeffs_a
      */
     if(coeffs_d){
         if(CAT(TYPE, _upsampling_convolution_valid_sf)(coeffs_d, input_len,
-                                                  wavelet->CAT(rec_hi_, TYPE),
+                                                  wavelet->CAT(rec_hi_, REAL_TYPE),
                                                   wavelet->rec_len, output,
                                                   output_len, mode) < 0){
             goto error;
@@ -441,12 +445,12 @@ int CAT(TYPE, _idwt)(const TYPE * const restrict coeffs_a, const size_t coeffs_a
 }
 
 /* basic SWT step (TODO: optimize) */
-int CAT(TYPE, _swt_)(TYPE input[], pywt_index_t input_len,
-                     const TYPE filter[], pywt_index_t filter_len,
-                     TYPE output[], pywt_index_t output_len,
+int CAT(TYPE, _swt_)(const TYPE * const restrict input, pywt_index_t input_len,
+                     const REAL_TYPE * const restrict filter, pywt_index_t filter_len,
+                     TYPE * const restrict output, size_t output_len,
                      unsigned int level){
 
-    TYPE * e_filter;
+    REAL_TYPE * e_filter;
     pywt_index_t i, e_filter_len, fstep;
     int ret;
 
@@ -463,7 +467,8 @@ int CAT(TYPE, _swt_)(TYPE input[], pywt_index_t input_len,
     if(level > 1){
         /* allocate filter first */
         e_filter_len = filter_len << (level-1);
-        e_filter = wtcalloc(e_filter_len, sizeof(TYPE));
+        if ((e_filter = wtcalloc(e_filter_len, sizeof(TYPE))) == NULL)
+            goto cleanup;
         if(e_filter == NULL)
             return -1;
         fstep = 1 << (level - 1);  // spacing between non-zero filter entries
@@ -483,30 +488,34 @@ int CAT(TYPE, _swt_)(TYPE input[], pywt_index_t input_len,
                                                     filter_len, output, 1,
                                                     1);
     }
+ cleanup:
+    wtfree(e_filter);
+    return -3;
 }
 
 /*
  * Approximation at specified level
  * input - approximation coeffs from upper level or signal if level == 1
  */
-int CAT(TYPE, _swt_a)(TYPE input[], pywt_index_t input_len,
-                      DiscreteWavelet* wavelet,
-                      TYPE output[], pywt_index_t output_len,
+int CAT(TYPE, _swt_a)(const TYPE * const restrict input, pywt_index_t input_len,
+                      const DiscreteWavelet * const restrict wavelet,
+                      TYPE * const restrict output, pywt_index_t output_len,
                       unsigned int level){
-    return CAT(TYPE, _swt_)(input, input_len, wavelet->CAT(dec_lo_, TYPE),
+    return CAT(TYPE, _swt_)(input, input_len, wavelet->CAT(dec_lo_, REAL_TYPE),
                             wavelet->dec_len, output, output_len, level);
 }
 
 /* Details at specified level
  * input - approximation coeffs from upper level or signal if level == 1
  */
-int CAT(TYPE, _swt_d)(TYPE input[], pywt_index_t input_len,
-                      DiscreteWavelet* wavelet,
-                      TYPE output[], pywt_index_t output_len,
+int CAT(TYPE, _swt_d)(const TYPE * const restrict input, pywt_index_t input_len,
+                      const DiscreteWavelet * const restrict wavelet,
+                      TYPE * const restrict output, pywt_index_t output_len,
                       unsigned int level){
-    return CAT(TYPE, _swt_)(input, input_len, wavelet->CAT(dec_hi_, TYPE),
+    return CAT(TYPE, _swt_)(input, input_len, wavelet->CAT(dec_hi_, REAL_TYPE),
                             wavelet->dec_len, output, output_len, level);
 }
 
+#endif /* REAL_TYPE */
 #endif /* TYPE */
 #undef restrict
